;;;-----------------------------LICENSE NOTICE------------------------------------
;;;    This file is part of LOOPED; An Amstrad CPC Game
;;;    Copyright (C) 2020  Miguel Ángel García Pérez (kupier@outlook.com) / Alejandro Gomariz Núñez (alejandro126g@gmail.com)
;;;
;;;    This program is free software: you can redistribute it and/or modify
;;;    it under the terms of the GNU General Public License as published by
;;;    the Free Software Foundation, either version 3 of the License, or
;;;    (at your option) any later version.
;;;
;;;    This program is distributed in the hope that it will be useful,
;;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;    GNU General Public License for more details.
;;;
;;;    You should have received a copy of the GNU General Public License
;;;------------------------------------------------------------------------------
;;;;
;;;;
;;;; component manager

.include "cmp/array_structure.h.s"
.include "cpct_functions.h.s"
.include "man/entity.h.s"
.include "cmp/entity.h.s"
.include "cmp/hud.h.s"
;.include "assets/assets.h.s"
.include "cpctelera.h.s"
.include "sys/cpct_collision.h.s"
.include "sys/render.h.s"


.module entity_manager


DefineComponentArrayStructure _entity, max_entities, DefineCmp_Entity_default


man_entity_getArray::
	ld	ix, #_entity_array
	ld	a, (_entity_num)
ret

man_entity_getArrayIX::
	ld	ix, #_entity_array
ret
man_entity_getArrayIY::
	ld	iy, #_entity_array
ret


man_entity_init::

xor	a
ld	(_entity_num), a

ld	hl, #_entity_array
ld	(_entity_pend), hl

ret



man_entity_new::

ld	hl, #_entity_num
inc (hl)


ld	hl, (_entity_pend)
ld 	d,	h
ld	e,	l
ld	bc, #sizeof_e
add	hl, bc
ld	(_entity_pend), hl

ret



man_entity_create::

	push 	hl
	call	man_entity_new

	ld__ixh_d
	ld__ixl_e

	pop hl
	ldir

	;call man_entity_collision_add
	ret

;;------------------MAN_ENTITY_FORALL-------------------------
;;Recibe en HL la posicion de memoria donde empieza cada funcion
;;para poder ejeuctar asi la de render etc.

man_entity_forall::
ld  (_ent_counter), a 
ld (_llamada), hl

ld	ix, #_entity_array
_loop:
ld  a, e_type(ix)
cp #e_type_invalid
jr z, _no_AI_ent

_AI_ent:
_llamada = . + 1
call _llamada


_no_AI_ent:

_ent_counter = . +1
ld  a, #0
dec a
ret z

ld  (_ent_counter),a

ld de, #sizeof_e
add ix, de

jr _loop

;;---------------------MAN_ENTITY_FORALL_MATCHING---------------
;;Recibe en HL lo mismo que en el forall
;;En a recibe una signature
man_entity_forall_matching:
	ld 	(ptrfunction),hl
	ld 	(signature),a
	ld 	ix,#_entity_array

_fam_next_entity:
	ld 	a,e_type(ix)
	or a
	ret z
	

signature = . + 1
	ld 	b,#00
	and b
	sub b
	jr 	nz,_fam_not_matching

ptrfunction = . + 1
	call ptrfunction
_fam_not_matching:
	ld bc,#sizeof_e
	add ix,bc
	jr _fam_next_entity



;---------------------MAN_ENTITY_FORALL_MATCHING_PAIRING---------------
;;Recibe en HL lo mismo que en el forall
;;En a recibe una signature
man_entity_forall_matching_pairing:
	ld 	(ptrfunctionpairs),hl
	ld 	(signaturepair),a
	ld 	ix,#_entity_array
	ld 	iy,#_entity_array
	ld 	bc,#sizeof_e
	add iy,bc

_fam_next_entity_pairing:
	ld 	a,e_type(ix)
	or a
	ret z
	

signaturepair = . + 1
	ld 	b,#00
	and b
	sub b
	jr 	nz,_fam_not_matching_pair

	ld 	a,e_type(iy)
	and b
	sub b
	jr 	nz,_fam_not_matching_pair

ptrfunctionpairs = . + 1
	call ptrfunctionpairs
_fam_not_matching_pair:
	ld bc,#sizeof_e
	add iy,bc
	ld 	a,e_type(iy)
	or 	a
	jr 	z,change_pr
	jr _fam_next_entity_pairing

change_pr:
	add ix,bc
	ld__d_ixh
	ld__e_ixl

	ld__iyh_d
	ld__iyl_e

	add iy,bc
	jr _fam_next_entity_pairing
;;----------------------------------------------------------------

man_entity_forall_hud:
ld 	(ptrfunctHud),hl
	ld	ix,#_hud
loop_hud:
	ld__d_ixh
	ld__e_ixl
	ld 	a,(de)
	inc a
	ret z;;Comprobamos si es FF pq cuando se le suma 1 da z
	ptrfunctHud = . + 1
	call ptrfunctHud
	jr loop_hud
ret

;;;--------------------MAN_ENTITY_4DESTRUCTION------------------

man_entity_4destruction::
	ld 	a,e_type(ix)
	or #e_type_dead
	ld 	e_type(ix),a

ret


;;------------------MAN_ENTTY_DESTROY---------------------------

man_entity_destroy::
;a mi me pasan por parametro la entidad que hay que destruir 
;que es de tipo dead	
	ld  e,e_lastVP_l(ix)
    ld  d,e_lastVP_h(ix)
    ld  a,#0xcc
    ld  c,e_w(ix)
    ld  b,e_h(ix)
    call cpct_drawSolidBox_asm

	ld 	hl,(_entity_pend)
	ld 	bc,#-sizeof_e
	add hl,bc
	ld 	(_entity_pend),hl

	ld__a_ixl
	sub l
	jr 	nz,_copy
	ld__a_ixh
	sub h
	jr 	z,_nocopy
_copy:
	ld__e_ixl
	ld__d_ixh
	ld 	bc,#sizeof_e
	ldir 
	ld 	bc,#-sizeof_e
	add hl,bc
	;copiar hl en de entonces de hay q ponerla a invalid
	
_nocopy:
	ld (hl),#e_type_invalid

ret
;;-----------MAN_ENTITY_DESTROY_ALL-------------------------
man_destroy_all_entities:
	ld 	a,(_entity_num)
	ld 	(cont_num),a
	cont_num = . + 1
	ld 	a,#0
	ld 	de,#_entity_array
bucle:
	ld 	a,#0
	ld 	bc,#sizeof_e
	call cpct_memset_asm
	ld 	a,(cont_num)
	dec a
	ret z
	ld 	(cont_num),a
	ex de,hl
	add hl,bc
	ex de,hl
	jr bucle
ret


;;-----------MAN_ENTITY_DESTROY_ALL-EXCEPT-HEROE------------------------
man_destroy_all_entities_ex_heroe: ;;No borra el personaje principal
	ld 	a,(_entity_num)
	dec a
	ld 	(cont_num2),a
	cont_num2 = . + 1
	ld 	a,#0
	ld 	de,#_entity_array
	ld 	bc,#sizeof_e
	ex de,hl
	add hl,bc
	ex de,hl ;;Asi aumentamos 
bucle_ex:
	ld 	bc,#sizeof_e
	ld 	a,#0
	call cpct_memset_asm
	ld 	a,(cont_num2)
	dec a
	ret z
	ld 	(cont_num2),a
	ex de,hl
	add hl,bc
	ex de,hl
	jr bucle_ex
ret

reset_values:
	ld	hl, #_entity_array
	ld 	bc,#sizeof_e
	add hl,bc
	ld	(_entity_pend), hl
	ld 	a,#1
	ld 	(_entity_num),a
	call man_entity_getArray
	ld 	e_type(ix),#e_type_MovAARenAAinputAAcollider
ret


;;-------------------MAN_ENTITY_UPDATE-----------------------


man_entity_update::
	ld 	a,(_entity_num)
	ld 	(contador),a
	ld 	ix,#_entity_array

_bucle:
	ld 	a,e_type(ix)
	and #e_type_dead
	sub #e_type_dead
	jr nz,not_to_destroy

	call man_entity_destroy

not_to_destroy:
	ld 	a,(contador)
	dec a
	ret z

	ld  (contador),a
	ld de, #sizeof_e
	add ix, de
jr _bucle

ret